from .country import Country as Country
from .currency import Currency as Currency
from .ltree import Ltree as Ltree
from .weekday import WeekDay as WeekDay
from .weekdays import WeekDays as WeekDays



from sqlalchemy_utils.functions.orm import (
    list_local_remote_exprs as list_local_remote_exprs,
    list_local_values as list_local_values,
    local_values as local_values,
    remote as remote,
    remote_column_names as remote_column_names,
    remote_values as remote_values,
)
from sqlalchemy_utils.generic import GenericRelationshipProperty as GenericRelationshipProperty
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from ..query_chain import QueryChain as QueryChain
from .database import has_index as has_index
from .orm import get_column_key as get_column_key, get_mapper as get_mapper, get_tables as get_tables
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from .. import ImproperlyConfigured as ImproperlyConfigured, i18n as i18n
from ..primitives import Currency as Currency
from .scalar_coercible import ScalarCoercible as ScalarCoercible
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from typing import Any

from .arrow import ArrowType as ArrowType
from .choice import Choice as Choice, ChoiceType as ChoiceType
from .color import ColorType as ColorType
from .country import CountryType as CountryType
from .currency import CurrencyType as CurrencyType
from .email import EmailType as EmailType
from .encrypted.encrypted_type import EncryptedType as EncryptedType, StringEncryptedType as StringEncryptedType
from .enriched_datetime.enriched_date_type import EnrichedDateType as EnrichedDateType
from .enriched_datetime.enriched_datetime_type import EnrichedDateTimeType as EnrichedDateTimeType
from .ip_address import IPAddressType as IPAddressType
from .json import JSONType as JSONType
from .locale import LocaleType as LocaleType
from .ltree import LtreeType as LtreeType
from .password import Password as Password, PasswordType as PasswordType
from .pg_composite import (
    CompositeType as CompositeType,
    register_composites as register_composites,
    remove_composite_listeners as remove_composite_listeners,
)
from .phone_number import (
    PhoneNumber as PhoneNumber,
    PhoneNumberParseException as PhoneNumberParseException,
    PhoneNumberType as PhoneNumberType,
)
from .range import (
    DateRangeType as DateRangeType,
    DateTimeRangeType as DateTimeRangeType,
    Int8RangeType as Int8RangeType,
    IntRangeType as IntRangeType,
    NumericRangeType as NumericRangeType,
)
from .scalar_list import ScalarListException as ScalarListException, ScalarListType as ScalarListType
from .timezone import TimezoneType as TimezoneType
from .ts_vector import TSVectorType as TSVectorType
from .url import URLType as URLType
from .uuid import UUIDType as UUIDType
from .weekdays import WeekDaysType as WeekDaysType
from sqlalchemy.orm.collections import InstrumentedList as _InstrumentedList

def __getattr__(name: str) -> Any: ...  # incomplete



from collections.abc import Mapping, Sequence

from sqlalchemy import Column, ForeignKeyConstraint
from sqlalchemy.engine import URL
from sqlalchemy.sql.functions import Function
from typing_extensions import TypeAlias

from ..utils import starts_with as starts_with
from .orm import quote as quote
from typing import Union, Optional, Any

# Should be `_VT: TypeAlias = Union[int, float, str, Mapping[str, "_VT"], Sequence["_VT"]]`
# But recursive types are not yet fully supported by mypy. See https://github.com/python/mypy/issues/731
_VT: TypeAlias = Union[int, float, str, Mapping[str, Any], Sequence[Any]]

def escape_like(string: str, escape_char: str = ...) -> str: ...
def json_sql(value: _VT, scalars_to_json: bool = ...) -> Function[Any]: ...
def jsonb_sql(value: _VT, scalars_to_jsonb: bool = ...) -> Function[Any]: ...
def has_index(column_or_constraint: Union[Column[Any], ForeignKeyConstraint]) -> bool: ...
def has_unique_index(column_or_constraint: Union[Column[Any], ForeignKeyConstraint]) -> bool: ...
def is_auto_assigned_date_column(column: Column[Any]) -> bool: ...
def database_exists(url: Union[URL, str]) -> bool: ...
def create_database(url: Union[URL, str], encoding: str = ..., template: Optional[str] = ...) -> None: ...
def drop_database(url: Union[URL, str]) -> None: ...



from ..exceptions import ImproperlyConfigured as ImproperlyConfigured
from .scalar_coercible import ScalarCoercible as ScalarCoercible
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from ..exceptions import ImproperlyConfigured as ImproperlyConfigured
from .enriched_datetime import ArrowDateTime as ArrowDateTime
from .enriched_datetime.enriched_datetime_type import EnrichedDateTimeType as EnrichedDateTimeType
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from .compat import get_scalar_subquery as get_scalar_subquery
from .functions.orm import get_column_key as get_column_key
from .relationships import (
    chained_join as chained_join,
    path_to_relationships as path_to_relationships,
    select_correlated_expression as select_correlated_expression,
)
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from sqlalchemy_utils.exceptions import ImproperlyConfigured as ImproperlyConfigured
from sqlalchemy_utils.types.encrypted.padding import PADDING_MECHANISM as PADDING_MECHANISM
from sqlalchemy_utils.types.json import JSONType as JSONType
from sqlalchemy_utils.types.scalar_coercible import ScalarCoercible as ScalarCoercible
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from .. import ImproperlyConfigured as ImproperlyConfigured, i18n as i18n
from ..utils import str_coercible as str_coercible
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from .functions import getdotattr as getdotattr, has_changes as has_changes
from .path import AttrPath as AttrPath
from .utils import is_sequence as is_sequence
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from .. import i18n as i18n
from ..exceptions import ImproperlyConfigured as ImproperlyConfigured
from ..primitives import WeekDay as WeekDay, WeekDays as WeekDays
from .bit import BitType as BitType
from .scalar_coercible import ScalarCoercible as ScalarCoercible
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from .database import (
    create_database as create_database,
    database_exists as database_exists,
    drop_database as drop_database,
    escape_like as escape_like,
    has_index as has_index,
    has_unique_index as has_unique_index,
    is_auto_assigned_date_column as is_auto_assigned_date_column,
    json_sql as json_sql,
    jsonb_sql as jsonb_sql,
)
from .foreign_keys import (
    dependent_objects as dependent_objects,
    get_fk_constraint_for_columns as get_fk_constraint_for_columns,
    get_referencing_foreign_keys as get_referencing_foreign_keys,
    group_foreign_keys as group_foreign_keys,
    merge_references as merge_references,
    non_indexed_foreign_keys as non_indexed_foreign_keys,
)
from .mock import create_mock_engine as create_mock_engine, mock_engine as mock_engine
from .orm import (
    cast_if as cast_if,
    get_bind as get_bind,
    get_class_by_table as get_class_by_table,
    get_column_key as get_column_key,
    get_columns as get_columns,
    get_declarative_base as get_declarative_base,
    get_hybrid_properties as get_hybrid_properties,
    get_mapper as get_mapper,
    get_primary_keys as get_primary_keys,
    get_tables as get_tables,
    get_type as get_type,
    getdotattr as getdotattr,
    has_changes as has_changes,
    identity as identity,
    is_loaded as is_loaded,
    naturally_equivalent as naturally_equivalent,
    quote as quote,
    table_name as table_name,
)
from .render import render_expression as render_expression, render_statement as render_statement
from .sort_query import make_order_by_deterministic as make_order_by_deterministic



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
Modified by Madoshakalaka@Github (dependency links added)
"""
from os import path

from setuptools import glob
from setuptools import setup

# Always prefer setuptools over distutils

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="types-sqlalchemy-utils",
    description="Type Stubs for sqlalchemy-utils",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cex-solutions/sqlalchemy-utils",
    author="Binovate Labs",
    author_email="cex-dev@binovate.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Typing :: Stubs Only",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    license="GPLv3",
    keywords="stubs sqlalchemy sqlalchemy-utils",
    package_data={
        "sqlalchemy_utils-stubs": [
            item.split("sqlalchemy_utils-stubs/")[-1] for item in glob.glob("**/*.pyi", recursive=True)
        ]
        + ["METADATA.toml"]
    },
    packages=["sqlalchemy_utils-stubs"],
    python_requires=">=3.7, <4",
    install_requires=[],
    extras_require={
        "dev": [
            "sqlalchemy-utils==0.38.2",
            "sqlalchemy2-stubs==0.0.2a21",
            "mypy==0.942",
            "pipenv-setup==3.2.0",
            "twine==4.0.0",
        ]
    },
    dependency_links=[],
    project_urls={
        "Bug Reports": "https://github.com/cex-solutions/types-sqlalchemy-utils/issues",
        "Source": "https://github.com/cex-solutions/types-sqlalchemy-utils",
    },
)



from ..scalar_coercible import ScalarCoercible as ScalarCoercible
from .pendulum_datetime import PendulumDateTime as PendulumDateTime
from typing import Any

def __getattr__(name: str) -> Any: ...  # incomplete



from .aggregates import aggregated as aggregated
from .asserts import (
    assert_max_length as assert_max_length,
    assert_max_value as assert_max_value,
    assert_min_value as assert_min_value,
    assert_non_nullable as assert_non_nullable,
    assert_nullable as assert_nullable,
)
from .exceptions import ImproperlyConfigured as ImproperlyConfigured
from .expressions import Asterisk as Asterisk, row_to_json as row_to_json
from .functions import (
    cast_if as cast_if,
    create_database as create_database,
    create_mock_engine as create_mock_engine,
    database_exists as database_exists,
    dependent_objects as dependent_objects,
    drop_database as drop_database,
    escape_like as escape_like,
    get_bind as get_bind,
    get_class_by_table as get_class_by_table,
    get_column_key as get_column_key,
    get_columns as get_columns,
    get_declarative_base as get_declarative_base,
    get_fk_constraint_for_columns as get_fk_constraint_for_columns,
    get_hybrid_properties as get_hybrid_properties,
    get_mapper as get_mapper,
    get_primary_keys as get_primary_keys,
    get_referencing_foreign_keys as get_referencing_foreign_keys,
    get_tables as get_tables,
    get_type as get_type,
    group_foreign_keys as group_foreign_keys,
    has_changes as has_changes,
    has_index as has_index,
    has_unique_index as has_unique_index,
    identity as identity,
    is_loaded as is_loaded,
    json_sql as json_sql,
    jsonb_sql as jsonb_sql,
    merge_references as merge_references,
    mock_engine as mock_engine,
    naturally_equivalent as naturally_equivalent,
    render_expression as render_expression,
    render_statement as render_statement,
    table_name as table_name,
)
from .generic import generic_relationship as generic_relationship
from .i18n import TranslationHybrid as TranslationHybrid
from .listeners import (
    auto_delete_orphans as auto_delete_orphans,
    coercion_listener as coercion_listener,
    force_auto_coercion as force_auto_coercion,
    force_instant_defaults as force_instant_defaults,
)
from .models import Timestamp as Timestamp, generic_repr as generic_repr
from .observer import observes as observes
from .primitives import (
    Country as Country,
    Currency as Currency,
    Ltree as Ltree,
    WeekDay as WeekDay,
    WeekDays as WeekDays,
)
from .proxy_dict import ProxyDict as ProxyDict, proxy_dict as proxy_dict
from .query_chain import QueryChain as QueryChain
from .types import (
    ArrowType as ArrowType,
    Choice as Choice,
    ChoiceType as ChoiceType,
    ColorType as ColorType,
    CompositeType as CompositeType,
    CountryType as CountryType,
    CurrencyType as CurrencyType,
    DateRangeType as DateRangeType,
    DateTimeRangeType as DateTimeRangeType,
    EmailType as EmailType,
    EncryptedType as EncryptedType,
    EnrichedDateTimeType as EnrichedDateTimeType,
    EnrichedDateType as EnrichedDateType,
    IPAddressType as IPAddressType,
    InstrumentedList as InstrumentedList,
    Int8RangeType as Int8RangeType,
    IntRangeType as IntRangeType,
    JSONType as JSONType,
    LocaleType as LocaleType,
    LtreeType as LtreeType,
    NumericRangeType as NumericRangeType,
    Password as Password,
    PasswordType as PasswordType,
    PhoneNumber as PhoneNumber,
    PhoneNumberParseException as PhoneNumberParseException,
    PhoneNumberType as PhoneNumberType,
    ScalarListException as ScalarListException,
    ScalarListType as ScalarListType,
    StringEncryptedType as StringEncryptedType,
    TSVectorType as TSVectorType,
    TimezoneType as TimezoneType,
    URLType as URLType,
    UUIDType as UUIDType,
    WeekDaysType as WeekDaysType,
    instrumented_list as instrumented_list,
    register_composites as register_composites,
    remove_composite_listeners as remove_composite_listeners,
)
from .view import (
    create_materialized_view as create_materialized_view,
    create_view as create_view,
    refresh_materialized_view as refresh_materialized_view,
)
